@@ -41,7 +41,7 @@ class WindowSelector(object):
         :param observed: The preprocessed, observed waveform.
         :type observed: :class:`~obspy.core.trace.Trace` or single component
             :class:`~obspy.core.stream.Stream`
-        :param observed: The preprocessed, synthetic waveform.
+        :param synthetic: The preprocessed, synthetic waveform.
         :type synthetic: :class:`~obspy.core.trace.Trace` or single component
             :class:`~obspy.core.stream.Stream`
         :param config: Configuration object.
@@ -201,9 +201,9 @@ class WindowSelector(object):
                 if not self.event.origins:
                     raise PyflexError("Event does not contain an origin.")
                 origin = self.event.preferred_origin() or self.event.origins[0]
-                self.event = Event(latitude=origin.latitude,
-                                   longitude=origin.longitude,
-                                   depth_in_m=origin.depth,
+                self.event = Event(latitude=float(origin.latitude),
+                                   longitude=float(origin.longitude),
+                                   depth_in_m=float(origin.depth),
                                    origin_time=origin.time)
             else:
                 raise PyflexError("Could not parse the event. Unknown type.")
@@ -213,7 +213,7 @@ class WindowSelector(object):
             net = self.observed.stats.network
             sta = self.observed.stats.station
             # Workaround for ObsPy 0.9.2 Newer version have a get
-            # coordiantes method...
+            # coordinates method...
             for network in self.station:
                 if network.code == net:
                     break
@@ -226,8 +226,8 @@ class WindowSelector(object):
             else:
                 raise PyflexError("Could not find the station of the "
                                   "observed data in the inventory object.")
-            self.station = Station(latitude=station.latitude,
-                                   longitude=station.longitude)
+            self.station = Station(latitude=float(station.latitude),
+                                   longitude=float(station.longitude))
 
         # Last resort, if either is not set, and the observed or synthetics
         # are sac files, get the information from there.
@@ -269,10 +269,51 @@ class WindowSelector(object):
                               self.config.min_period)
         self.peaks, self.troughs = utils.find_local_extrema(self.stalta)
 
+        if not len(self.peaks) and len(self.troughs):
+            return
+
+        if self.ttimes:
+            offset = self.event.origin_time - self.observed.stats.starttime
+            min_time = self.ttimes[0]["time"] - \
+                self.config.max_time_before_first_arrival + offset
+            min_idx = int(min_time / self.observed.stats.delta)
+
+            dist_in_km = geodetics.calcVincentyInverse(
+                self.station.latitude, self.station.longitude,
+                self.event.latitude, self.event.longitude)[0] / 1000.0
+            max_time = dist_in_km / self.config.min_surface_wave_velocity + \
+                offset + self.config.max_period
+            max_idx = int(max_time / self.observed.stats.delta)
+
+            # Reject all peaks and troughs before the minimal allowed start
+            # time and after the maximum allowed end time.
+            first_trough, last_trough = self.troughs[0], self.troughs[-1]
+            self.troughs = self.troughs[(self.troughs >= min_idx) &
+                                        (self.troughs <= max_idx)]
+
+            # If troughs have been removed, readd them add the boundaries.
+            if len(self.troughs):
+                if first_trough != self.troughs[0]:
+                    self.troughs = np.concatenate([
+                        np.array([min_idx], dtype=self.troughs.dtype),
+                        self.troughs])
+                if last_trough != self.troughs[-1]:
+                    self.troughs = np.concatenate([
+                        self.troughs,
+                        np.array([max_idx], dtype=self.troughs.dtype)])
+            # Make sure peaks are inside the troughs!
+            min_trough, max_trough = self.troughs[0], self.troughs[-1]
+            self.peaks = self.peaks[(self.peaks > min_trough) &
+                                    (self.peaks < max_trough)]
+
     def select_windows(self):
         """
         Launch the window selection.
         """
+        # Fill self.ttimes.
+        if self.event and self.station:
+            self.calculate_ttimes()
+
         self.calculate_preliminiaries()
 
         # Perform all window selection steps.
@@ -280,7 +321,6 @@ class WindowSelector(object):
         # Reject windows based on traveltime if event and station
         # information is given. This will also fill self.ttimes.
         if self.event and self.station:
-            self.calculate_ttimes()
             self.reject_on_traveltimes()
         else:
             msg = "No rejection based on traveltime possible. Event and/or " \
@@ -379,14 +419,29 @@ class WindowSelector(object):
 
         noise = self.observed.data[self.config.noise_start_index:
                                    self.config.noise_end_index]
-        noise_amp = np.abs(noise).max()
 
-        def filter_window_noise(win):
-            win_signal = self.observed.data[win.left: win.right]
-            win_noise_amp = np.abs(win_signal).max() / noise_amp
-            if win_noise_amp < self.config.s2n_limit[win.center]:
-                return False
-            return True
+        if self.config.window_signal_to_noise_type == "amplitude":
+            noise_amp = np.abs(noise).max()
+
+            def filter_window_noise(win):
+                win_signal = self.observed.data[win.left: win.right]
+                win_noise_amp = np.abs(win_signal).max() / noise_amp
+                if win_noise_amp < self.config.s2n_limit[win.center]:
+                    return False
+                return True
+
+        elif self.config.window_signal_to_noise_type == "energy":
+            noise_energy = np.sum(noise ** 2) / len(noise)
+
+            def filter_window_noise(win):
+                data = self.observed.data[win.left: win.right]
+                win_energy = np.sum(data ** 2) / len(data)
+                win_noise_amp = win_energy / noise_energy
+                if win_noise_amp < self.config.s2n_limit[win.center]:
+                    return False
+                return True
+        else:
+            raise NotImplementedError
 
         self.windows = list(filter(filter_window_noise, self.windows))
         logger.info("SN amplitude ratio window rejection retained %i windows" %
@@ -692,10 +747,16 @@ class WindowSelector(object):
 
             if not (tshift_min < win.cc_shift *
                     self.observed.stats.delta < tshift_max):
+                logger.debug("Window rejected due to time shift: %f" %
+                             win.cc_shift)
                 return False
             if not (dlnA_min < win.dlnA < dlnA_max):
+                logger.debug("Window rejected due to amplitude fit: %f" %
+                             win.dlnA)
                 return False
             if win.max_cc_value < self.config.cc_acceptance_level[win.center]:
+                logger.debug("Window rejected due to CC value: %f" %
+                             win.max_cc_value)
                 return False
             return True
 
